package OSOP;

import java.awt.Color;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Timer;

import org.eclipse.core.resources.IProject;
import org.eclipse.jface.action.ActionContributionItem;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.events.MenuEvent;
import org.eclipse.swt.events.MenuListener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.views.navigator.IResourceNavigator;

public class MenuExecutor {
	private IResourceNavigator navigator;
	private IProject[] projects;
	private String[] commandList;
	private String[] commandListKorean;
	private MenuExecutor next;
	private MenuExecutor parent;
	private StringBuffer message = new StringBuffer();
	private String viewName;
	private long elapsedMillis;
	private int index;

	public MenuExecutor() {
	}

	public String toString() {
		String value = this.commandListToString();
		if (this.getProjectName() != null) {
			value += " \"" + this.getProjectName() + "\"";
		}
		return value;
	}

	public String getElpasedTime(long milli) {
		if (milli < 1000) {
			return milli + "ms";
		} else if (milli < 60 * 1000) {
			long currentMilli = milli % 1000;
			long currentSec = milli / 1000;
			String value = currentSec + "sec" + (currentSec > 1 ? "s" : "");

			if (currentMilli > 0) {
				value += " " + this.getElpasedTime(currentMilli);
			}
			return value;
		} else {
			long currentMillis = milli % (60 * 1000);
			long currentMin = milli / (60 * 1000);
			String value = currentMin + "min" + (currentMin > 1 ? "s" : "");
			if (currentMillis > 0) {
				value += " " + this.getElpasedTime(currentMillis);
			}
			return value;
		}
	}

	public String getProjectName() {
		if (this.projects == null) {
			return null;
		}

		if (this.projects.length == 0) {
			return null;
		} else if (projects.length == 1) {
			return this.projects[0].getName();
		} else {
			return this.projects.length + " projects";
		}
	}

	public String replace(String value, String src, String dest) {
		if (value == null) {
			return null;
		}

		int start = 0;
		int end = 0;
		StringBuffer buffer = new StringBuffer();

		boolean isOK = true;
		while (isOK) {
			end = value.indexOf(src, start);
			if (end == -1) {
				buffer.append(value.substring(start));
				isOK = false;
			} else {
				if (end >= start) {
					buffer.append(value.substring(start, end));
					buffer.append(dest);
					start = end + src.length();
				}
			}
		}

		return buffer.toString();
	}

	public String commandListToString() {
		if (this.commandList == null) {
			return null;
		}

		String returnValue = null;
		if (this.commandList.length == 0) {
			returnValue = null;
		} else if (commandList.length == 1) {
			returnValue = this.commandList[0];
		} else {
			StringBuffer buffer = new StringBuffer();
			for (int i = 0; i < this.commandList.length; i++) {
				buffer.append(commandList[i]);
				if (i != (this.commandList.length - 1)) {
					buffer.append("->");
				}
			}
			returnValue = buffer.toString();
		}
		return replace(returnValue, "&", "");
	}

	public void execute(final ProgressBarFrame progress) {
		progress.setValueAndMessage(
			Color.black,
			index,
			0,
			"starting...",
			"* " + MenuExecutor.this.toString() + " starting...");
		navigator.getViewer().setSelection(new StructuredSelection(projects));
		final Menu menu = navigator.getViewer().getTree().getMenu();
		menu.setVisible(true);
		menu.addMenuListener(new MenuListener() {
			public void menuHidden(MenuEvent e) {
			}

			public void menuShown(MenuEvent e) {
				long start = System.currentTimeMillis();
				menu.removeMenuListener(this);
				menu.setVisible(false);

				try {
					if (viewName != null) {
						PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().showView(viewName);
					}
				} catch (Exception ex) {
				}

				Timer timer = new Timer(1000, new ActionListener() {
					int sec = 1;
					public void actionPerformed(ActionEvent arg0) {
						new Thread() {
							public void run() {
								progress.setValueAndMessage(
									Color.black,
									index,
									sec % 60,
									MenuExecutor.this.getElpasedTime(1000 * sec),
									"* "
										+ MenuExecutor.this.toString()
										+ " running... ["
										+ getElpasedTime(1000 * sec++)
										+ "]");
							}
						}
						.start();
					}
				});
				timer.start();
				execute(menu, commandList);
				elapsedMillis = System.currentTimeMillis() - start;
				timer.stop();
				progress.setValueAndMessage(
					new Color(102, 102, 153),
					index,
					0,
					MenuExecutor.this.getElpasedTime(),
					MenuExecutor.this.toString() + " done. [" + MenuExecutor.this.getElpasedTime() + "]");

				message.append(MenuExecutor.this.toString() + ":" + getElpasedTime(elapsedMillis));
				Log.log(message.toString());
				Log.log(MenuExecutor.this.toString() + ">>>" + next);
				if (next != null) {
					next.execute(progress);
				} else {
					progress.dispose();

					StringBuffer log = new StringBuffer();
					MenuExecutor parent = MenuExecutor.this;
					long totalElpasedMilli = parent.getElapsedMillis();
					log.insert(0, parent.getMessage());
					while ((parent = parent.getParent()) != null) {
						log.insert(0, parent.getMessage() + "\r\n");
						totalElpasedMilli += parent.getElapsedMillis();
					}
					log.append("\r\n\r\n");
					log.append("Total elapsed time is " + MenuExecutor.this.getElpasedTime(totalElpasedMilli));
					MessageDialog.openInformation(
						PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell(),
						"OSOP",
						log.toString());
				}
			}
		});
	}

	private void execute(Menu menu, String[] commandList) {
		if (menu == null) {
			return;
		}

		if (commandList == null) {
			return;
		}

		MenuItem[] items = menu.getItems();
		for (int i = 0; i < commandList.length; i++) {
			for (int j = 0; j < items.length; j++) {
				Log.log("menu name is " + items[j].getText());
				if (commandList[i].equals(items[j].getText()) || commandListKorean[i].equals(items[j].getText())) {
					if (i == (commandList.length - 1)) {
						ActionContributionItem actionItem = (ActionContributionItem) (items[j].getData());
						Log.log(actionItem.getAction().getText() + " run");
						actionItem.getAction().run();
						return;
					} else {
						items = items[j].getMenu().getItems();
					}
					break;
				}
			}
		}
	}

	public String[] getCommandList() {
		return commandList;
	}

	public MenuExecutor getNext() {
		return next;
	}

	public IProject[] getProjects() {
		return projects;
	}

	public IResourceNavigator getNavigator() {
		return navigator;
	}

	public void setCommandList(String[] strings) {
		commandList = strings;
	}

	public void setNext(MenuExecutor executor) {
		executor.parent = this;
		next = executor;
	}

	public void setProjects(IProject[] projects) {
		this.projects = projects;
	}

	public void setNavigator(IResourceNavigator navigator) {
		this.navigator = navigator;
	}

	public String getMessage() {
		return this.message.toString();
	}

	public MenuExecutor getParent() {
		return parent;
	}

	public String getViewName() {
		return viewName;
	}

	public void setViewName(String string) {
		viewName = string;
	}

	public String[] getCommandListKorean() {
		return commandListKorean;
	}

	public void setCommandListKorean(String[] strings) {
		commandListKorean = strings;
	}

	public long getElapsedMillis() {
		return elapsedMillis;
	}

	public String getElpasedTime() {
		return this.getElpasedTime(this.elapsedMillis);
	}

	public void setElapsedMillis(long l) {
		elapsedMillis = l;
	}

	public int getIndex() {
		return index;
	}

	public void setIndex(int i) {
		index = i;
	}
}
