package OSOP;

import java.io.File;
import java.util.Vector;

import org.eclipse.core.internal.runtime.InternalPlatform;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IPageLayout;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.views.navigator.IResourceNavigator;

public class OSOPAction implements IWorkbenchWindowActionDelegate {

	public static final String MESSAGE_DIALOG_TITLE = "OSOP";
	
	public void dispose() {
	}

	public void init(IWorkbenchWindow window) {
	}

	public void run(IAction action) {
		Log.setLogFile(new File(InternalPlatform.getMetaArea().getLocation().toFile(), "osop.log"));
		Shell shell = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell();
		
		try {
			final IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
			final IResourceNavigator navigator = (IResourceNavigator) page.showView(IPageLayout.ID_RES_NAV);

			navigator.getViewer().collapseAll();
			Log.log("collapse all");
			IProject[] projects = ResourcesPlugin.getWorkspace().getRoot().getProjects();
			Log.log("get all project");

			Vector projectVector = new Vector();
			for (int i = 0; i < projects.length; i++) {
				Log.log(projects[i].getName());
				IFolder folder = projects[i].getFolder(".svn");
				if (folder.exists()) {
					Log.log("get svn project: " + projects[i].getName());
					projectVector.add(projects[i]);
				}
			}
			
			MenuExecutorChaine executorChain = new MenuExecutorChaine();
			MenuExecutor svnUpdate = new MenuExecutor();
			svnUpdate.setNavigator(navigator);
			svnUpdate.setProjects((IProject[]) projectVector.toArray(new IProject[0]));
			svnUpdate.setCommandList(new String[] { "T&eam", "&Update" });
			svnUpdate.setCommandListKorean(new String[] { "(&E)", "&Update" });
			svnUpdate.setViewName("org.tigris.subversion.subclipse.ui.console.ConsoleView");
			executorChain.add(svnUpdate);
			
			MenuExecutor miltilibBuild = new MenuExecutor();
			miltilibBuild.setNavigator(navigator);
			miltilibBuild.setProjects(getProject("milti-lib"));
			miltilibBuild.setCommandList(new String[] { "&Build Project" });
			miltilibBuild.setCommandListKorean(new String[] { "Ʈ (&B)" });
			executorChain.add(miltilibBuild);
			
			MenuExecutor miltiBuild = new MenuExecutor();
			miltiBuild.setNavigator(navigator);
			miltiBuild.setProjects(getProject("milti"));
			miltiBuild.setCommandList(new String[] { "Generate Deployment Code" });
			miltiBuild.setCommandListKorean(new String[] { " ڵ " });
			executorChain.add(miltiBuild);

			MenuExecutor webappBuild = new MenuExecutor();
			webappBuild.setNavigator(navigator);
			webappBuild.setProjects(getProject("webapp"));
			webappBuild.setCommandList(new String[] { "&Build Project" });
			webappBuild.setCommandListKorean(new String[] { "Ʈ (&B)" });
			executorChain.add(webappBuild);

			String confirmMessage = "Tasks of OSOP are below:\r\n";

			for (int i = 0 ; i < executorChain.size(); i++) {
				confirmMessage += " - " + executorChain.get(i).toString() + "\r\n";
			}
			confirmMessage += "\r\n";
			confirmMessage += "Start OSOP?";
			if (!MessageDialog.openConfirm(shell, MESSAGE_DIALOG_TITLE, confirmMessage)) {
				return;
			}

			ProgressBarFrame progress = new ProgressBarFrame(executorChain);		
			executorChain.get(0).execute(progress);
			
		} catch (Exception e) {
			MessageDialog.openInformation(shell, MESSAGE_DIALOG_TITLE, e.getMessage());
		}
	}

	public void selectionChanged(IAction action, ISelection selection) {
	}
	
	
	public IProject[] getProject(String projectName) {
		return new IProject[] { ResourcesPlugin.getWorkspace().getRoot().getProject(projectName)};
	}
}
